@@ -139,9 +139,13 @@ enum tp_gesture_state {
 };
 
 enum tp_thumb_state {
-	THUMB_STATE_NO,
-	THUMB_STATE_YES,
-	THUMB_STATE_MAYBE,
+	THUMB_STATE_FINGER,
+	THUMB_STATE_JAILED,
+	THUMB_STATE_PINCH,
+	THUMB_STATE_SUPPRESSED,
+	THUMB_STATE_REVIVED,
+	THUMB_STATE_REVIVED_JAILED,
+	THUMB_STATE_DEAD,
 };
 
 enum tp_jump_state {
@@ -237,12 +241,6 @@ struct tp_touch {
 		struct device_coords initial;
 	} gesture;
 
-	struct {
-		enum tp_thumb_state state;
-		uint64_t first_touch_time;
-		struct device_coords initial;
-	} thumb;
-
 	struct {
 		double last_speed; /* speed in mm/s at last sample */
 		unsigned int exceeded_count;
@@ -457,6 +455,10 @@ struct tp_dispatch {
 
 		bool use_size;
 		int size_threshold;
+
+		enum tp_thumb_state state;
+		unsigned int index;
+		bool pinch_eligible;
 	} thumb;
 
 	struct {
@@ -487,8 +489,12 @@ struct tp_dispatch {
 	} tablet_mode_switch;
 
 	struct {
-		/* true if the axes need rotation when left-handed is on*/
 		bool rotate;
+		bool want_rotate;
+
+		bool must_rotate; /* true if we should rotate when applicable */
+		struct evdev_device *tablet_device;
+		bool tablet_left_handed_state;
 	} left_handed;
 };
 
@@ -565,6 +571,10 @@ tp_filter_motion_unaccelerated(struct tp_dispatch *tp,
 bool
 tp_touch_active(const struct tp_dispatch *tp, const struct tp_touch *t);
 
+bool
+tp_touch_active_for_gesture(const struct tp_dispatch *tp,
+			    const struct tp_touch *t);
+
 int
 tp_tap_handle_state(struct tp_dispatch *tp, uint64_t time);
 
@@ -673,4 +683,34 @@ tp_palm_tap_is_palm(const struct tp_dispatch *tp, const struct tp_touch *t);
 void
 tp_clickpad_middlebutton_apply_config(struct evdev_device *device);
 
+bool
+tp_thumb_ignored(const struct tp_dispatch *tp, const struct tp_touch *t);
+
+void
+tp_thumb_reset(struct tp_dispatch *tp);
+
+bool
+tp_thumb_ignored_for_gesture(const struct tp_dispatch *tp, const struct tp_touch *t);
+
+bool
+tp_thumb_ignored_for_tap(const struct tp_dispatch *tp,
+			 const struct tp_touch *t);
+
+void
+tp_thumb_suppress(struct tp_dispatch *tp, struct tp_touch *t);
+
+void
+tp_thumb_update_touch(struct tp_dispatch *tp,
+		      struct tp_touch *t,
+		      uint64_t time);
+
+void
+tp_detect_thumb_while_moving(struct tp_dispatch *tp);
+
+void
+tp_thumb_update_multifinger(struct tp_dispatch *tp);
+
+void
+tp_init_thumb(struct tp_dispatch *tp);
+
 #endif
