@@ -35,16 +35,19 @@
 #include "litest.h"
 
 static void
-test_relative_event(struct litest_device *dev, int dx, int dy)
+test_relative_event(struct litest_device *dev, double dx, double dy)
 {
 	struct libinput *li = dev->libinput;
 	struct libinput_event_pointer *ptrev;
 	struct libinput_event *event;
+	struct udev_device *ud;
 	double ev_dx, ev_dy;
 	double expected_dir;
 	double expected_length;
 	double actual_dir;
 	double actual_length;
+	const char *prop;
+	int dpi = 1000;
 
 	litest_event(dev, EV_REL, REL_X, dx);
 	litest_event(dev, EV_REL, REL_Y, dy);
@@ -55,6 +58,23 @@ test_relative_event(struct litest_device *dev, int dx, int dy)
 	event = libinput_get_event(li);
 	ptrev = litest_is_motion_event(event);
 
+	/* low-dpi devices scale up, not down, especially for slow motion.
+	 * so a 1 unit movement in a 200dpi mouse still sends a 1 pixel
+	 * movement. Work aorund this here by checking for the MOUSE_DPI
+	 * property.
+	 */
+	ud = libinput_device_get_udev_device(dev->libinput_device);
+	litest_assert_ptr_notnull(ud);
+	prop = udev_device_get_property_value(ud, "MOUSE_DPI");
+	if (prop) {
+		dpi = parse_mouse_dpi_property(prop);
+		ck_assert_int_ne(dpi, 0);
+
+		dx *= 1000.0/dpi;
+		dy *= 1000.0/dpi;
+	}
+	udev_device_unref(ud);
+
 	expected_length = sqrt(4 * dx*dx + 4 * dy*dy);
 	expected_dir = atan2(dx, dy);
 
@@ -64,11 +84,11 @@ test_relative_event(struct litest_device *dev, int dx, int dy)
 	actual_dir = atan2(ev_dx, ev_dy);
 
 	/* Check the length of the motion vector (tolerate 1.0 indifference). */
-	litest_assert(fabs(expected_length) >= actual_length);
+	litest_assert_double_ge(fabs(expected_length), actual_length);
 
 	/* Check the direction of the motion vector (tolerate 2π/4 radians
 	 * indifference). */
-	litest_assert(fabs(expected_dir - actual_dir) < M_PI_2);
+	litest_assert_double_lt(fabs(expected_dir - actual_dir), M_PI_2);
 
 	libinput_event_destroy(event);
 
@@ -220,7 +240,7 @@ test_absolute_event(struct litest_device *dev, double x, double y)
 	litest_assert_int_eq(libinput_event_get_type(event), type);
 
 	ptrev = libinput_event_get_pointer_event(event);
-	litest_assert(ptrev != NULL);
+	litest_assert_ptr_notnull(ptrev);
 
 	ex = libinput_event_pointer_get_absolute_x_transformed(ptrev, 100);
 	ey = libinput_event_pointer_get_absolute_y_transformed(ptrev, 100);
@@ -781,8 +801,8 @@ END_TEST
 
 START_TEST(pointer_seat_button_count)
 {
-	const int num_devices = 4;
-	struct litest_device *devices[num_devices];
+	struct litest_device *devices[4];
+	const int num_devices = ARRAY_LENGTH(devices);
 	struct libinput *libinput;
 	struct libinput_event *ev;
 	struct libinput_event_pointer *tev;
